/* -*- mode: c; -*- */
/* Scanner for grammar files */

%option noyywrap
%option yylineno

%{
#include "scanner.h"

/* Which start condition entered comment */
static int comment_caller = 0;
%}

%x comment token_def start_def production

TOKEN_DEF           ^[[:blank:]]*%token
START_DEF           ^[[:blank:]]*%start
ID                  [[:alpha:]_]+[[:alnum:]_]*
/*
 * Define a charachter token type which can be any of the following:
 *     null
 *     alert (BEL)
 *     backspace
 *     form feed
 *     new line
 *     carriage return
 *     horizontal tab
 *     vertical tab
 *     backslash
 *     question mark
 *     single quote
 *     double quote
 *     byte with octal code
 *     byte with hexademical code
 *     other (`'X'', where X is not a new line, a backslash, or a qoute)
 */
CHAR_TTYPE          '\\0'|'\\a'|'\\b'|'\\f'|'\\n'|'\\r'|'\\t'|'\\v'|'\\\\'|'\\\?'|'\\\''|'\\\"'|'\\[[:digit:]]{3}'|'\\x[[:digit:]]{2}'|'[^\\\n\r\']'



%%



<comment>{
  [^*]*             /* eat up anything that's not a '*' */
  "*"+[^*/]*        /* eat up '*'s not followed by '/'s */
  "*"+"/"           BEGIN(comment_caller); /* end of comment */
  <<EOF>>           return SCANNED_ERR_UNTERM_COMMENT;
}



{TOKEN_DEF}         BEGIN(token_def); /* start of named token type definition */
<token_def>{
  "/*"              { /* Start of comment */
                      comment_caller = token_def;
                      BEGIN(comment); }
  {ID}              return SCANNED_NAMED_TOKEN_TYPE_DEF;
  {CHAR_TTYPE}      /* silently ignore char token types here */
  "\n"              BEGIN(INITIAL);   /* end of named token type definition */
}



{START_DEF}         BEGIN(start_def); /* start of start ID definition */
<start_def>{
  "/*"              { /* Start of comment */
                      comment_caller = start_def;
                      BEGIN(comment); }
  {ID}              return SCANNED_START_ID_DEF;
  {CHAR_TTYPE}      return SCANNED_ERR_CHAR_TTYPE_AS_START;
  "\n"              BEGIN(INITIAL);   /* end of start identifier definition */
}



{ID}:               { *(yytext + --yyleng) = '\0';  /* remove trailing `:' */
                      BEGIN(production);
                      return SCANNED_PRODUCTION_HEAD; }
{CHAR_TTYPE}:       { *(yytext + --yyleng) = '\0';  /* remove trailing `:' */
                      return SCANNED_ERR_CHAR_TTYPE_AS_PRODUCTION_HEAD; }
<production>{
  "/*"              { /* Start of comment */
                      comment_caller = production;
                      BEGIN(comment); }
  {ID}              return SCANNED_PBODY_IDENTIFIER;
  {CHAR_TTYPE}      return SCANNED_PBODY_CHAR_TOKEN_TYPE;
  \|                return SCANNED_PBODY_PIPE;
  ;                 { /* End of production */
                      BEGIN(INITIAL);
                      return SCANNED_PRODUCTION_END;
                    }
  <<EOF>>           return SCANNED_ERR_UNTERM_PRODUCTION;
}



"/*"                { /* Start of comment */
                      comment_caller = INITIAL;
                      BEGIN(comment); }

<INITIAL,token_def,start_def,production>[[:blank:]]+  /* don't skip blanks in
             comment start condition, for performance reasons--there skipping is
             more drastic than just skipping blanks, and will be handled by its
             own rules (see above) */

<INITIAL,production>"\n"+ /* skip newlines */

<INITIAL,token_def,start_def,production>. return SCANNED_ERR;



%%



/*
 * Local variables:
 * compile-command: "make -k && ./unreachable < grammar.3"
 * End:
 */
