#include "tools.h"

#include <stdio.h>  /* fprintf() */
#include <getopt.h>
#include <ctype.h>
#include <assert.h>


extern Grammar *remove_units(Grammar *grammar);


static char *in_file_name  = NULL;
static FILE *in_stream     = NULL;

static char *out_file_name = NULL;
static FILE *out_stream    = NULL;

char *progname = NULL;

static Grammar    *grammar  = NULL;
static const char *errOpenFileRead ="%s: cannot open file `%s' for reading\n";
static const char *errOpenFileWrite="%s: cannot open file `%s' for writing\n";
static const char *warnEmptyGrammar="%s: warning: input specifies empty grammar\n";

static int parse_options(int argc, char **argv);



int main(int argc, char **argv)
{
    int i;
    Symb *symb;

    /* Process command line arguments */
    progname = *argv;
    i = parse_options(argc, argv);
    if ( i == 0 )
        return 1;
    else if ( i == 1 )
        return 0;

    /* Set up the streams */
    if ( !in_file_name )
        in_stream = stdin;
    else {
        if ( !(in_stream = fopen(in_file_name, "r")) ) {
            fprintf(stderr, errOpenFileRead, progname, in_file_name);
            exit(1);
        }
    }
    if ( !out_file_name )
        out_stream = stdout;
    else {
        if ( !(out_stream = fopen(out_file_name, "w")) ) {
            fprintf(stderr, errOpenFileWrite, progname, out_file_name);
            exit(1);
        }
    }

    grammar = read_grammar(in_stream);
    if ( !grammar->head ) {  /* empty grammar--nothing to do */
        fprintf(stderr, warnEmptyGrammar, progname);
        exit(0);
    }






/*     fputs("Here is the alphabet I have:\n", out_stream); */

/*     symb = grammar->alphabet->head; */
/*     while ( symb ) { */
/*         fprintf(out_stream, "%s%s `%s'\n", symb->terminal ? "" : "non", "terminal", symb->name); */
/*         symb = symb->next; */
/*     } */

/*     fputs("\n\nHere is the grammar I have:\n", out_stream); */
/*     grammar_print(grammar, out_stream); */


    {
        Grammar *grammar_out;

/*         fputs("\n\nHere are unit pairs I figured:\n", out_stream); */
        grammar_out = remove_units(grammar);

/*         fputs("\n\nHere is resulting grammar:\n", out_stream); */
        grammar_print(grammar_out, out_stream);
    }

    return 0;
}


static char *errUnknownOption = "%s: error--unknown option `-%c'\n";
static char *errUnknownOptionChar = "%s: error--unknown option character `\\x%x'\n";
static char *errArgumentReq   = "%s: error--option `-%c' requires an argument (%s)\n";

static char *usehelp = "Try `%s -h' for help\n";
static char *usage   = "\n\
Usage: %s [-h] [-i <file_in>] [-o <file_out>]\n\
\n\
Eliminates unit productions from a grammar.\n\
\n\
-h              display this help message\n\
-i <file_in>    read grammar from file <file_in>, `-' for stdin (default)\n\
-o <file_out>   write grammar to file <file_out>, `-' for stdout (default)\n\
\n\
Notes:  The format of input and output grammars is similar to that of YACC.\n\
        The program does not recognize literal string tokens.  In addition to\n\
        `%%token' declaration there is a `%%start' declaration for specifying\n\
        the start symbol (identifier) of the grammar.  Note that when not\n\
        explicitly specified, head of the first production will be assumed to\n\
        be the start symbol.  Check out sample grammars for examples of usage.\n\
\n";

/*
 * Return codes:
 *     0    - error, exit(1);
 *     1    - ok, exit(0);
 *     else - ok, go on.
 */
static int parse_options(int argc, char **argv)
{
    char c;

    opterr = 0; /* prevent getopt from printing error messages */
    while ( (c = getopt(argc, argv, ":i:o:h")) != -1 )
        switch ( c ) {
        case 'i':
            if ( *optarg == '-' && *(optarg+1) == '\0' )
                in_file_name = NULL;
            else
                in_file_name = optarg;
            break;
        case 'o':
            if ( *optarg == '-' && *(optarg+1) == '\0' )
                out_file_name = NULL;
            else
                out_file_name = optarg;
            break;
        case 'h':
            printf(usage, progname);
            return 1;
            break;
        case '?': /* unknown option */
            if ( isprint(optopt) ) {
                fprintf(stderr, errUnknownOption, progname, optopt);
                fprintf(stderr, usehelp, progname);
            }
            else {
                fprintf(stderr, errUnknownOptionChar, progname, optopt);
                fprintf(stderr, usehelp, progname);
            }
            return 0;
            break;
        case ':': { /* missing option argument */
            char *msg = NULL;

            switch ( optopt ) {
            case 'i':
                msg = "input file name";
                break;
            case 'o':
                msg = "output file name";
                break;
            default:
                assert(0); /* should never get here */
                abort();
                break;
            }
            fprintf(stderr, errArgumentReq, progname, optopt, msg);
            fprintf(stderr, usehelp, progname);
            return 0;
            break;
        }
        default:
            assert(0); /* should never happen */
            abort();
            break;
        }

    return -1;
}



/*
 * Local variables:
 * compile-command: "make -k && ./units < grammar.4"
 * End:
 */
