/* Reverse polish notation calculator. */

%{
#define YYSTYPE double
#include <math.h>
#include <stdio.h>

int yylex (void);
void yyerror (const char *s);
%}

%token NUM

%% /* Grammar rules and actions follow */

input:    /* empty */
        | input line
;

line:     '\n'
        |  exp '\n'  { printf ("\t%.10g\n", $1); }
;

exp:      NUM         { $$ = $1;      }
        | exp exp '+' { $$ = $1 + $2; }
        | exp exp '-' { $$ = $1 - $2; }
        | exp exp '*' { $$ = $1 * $2; }
        | exp exp '/' { $$ = $1 / $2; }
      /* Exponentiation */
        | exp exp '^' { $$ = pow($1, $2); }
      /* Unary minus    */
        | exp 'n' { $$ = -$1; }
;
%%

/* Lexical analyzer returns a double floating point number on the stack and the
token NUM, or the ASCII character read if not a number.  Skips all blanks and
tabs, returns 0 for EOF. */

#include <ctype.h>

int yylex (void)
{
    int c;

    /* skip white space  */
    while ((c = getchar ()) == ' ' || c == '\t')
        ;
    /* process numbers   */
    if (c == '.' || isdigit (c)) {
        ungetc (c, stdin);
        scanf("%lf", &yylval);
        return NUM;
    }
    /* return end-of-file  */
    if (c == EOF)
        return 0;
    /* return single chars */
    return c;
}

void yyerror (const char *s)  /* Called by yyparse on error */
{
    printf ("%s\n", s);
}

int main (void)
{
    return yyparse();
}
